@@ -24,23 +24,29 @@ def unprotected_read_json_as_dict(db_name: str) -> dict:
 	json_path, json_exists, ddb_path, ddb_exists = db_paths(db_name)
 
 	if json_exists and ddb_exists:
-		raise Exception(f"DB Inconsistency: \"{db_name}\" exists as .json and .ddb")
+		raise FileExistsError(f"DB Inconsistency: \"{db_name}\" exists as .json and .ddb")
 
 	if not json_exists and not ddb_exists:
-		raise Exception(f"DB \"{db_name}\" does not exist.")
+		raise FileNotFoundError(f"DB \"{db_name}\" does not exist.")
+
+
+	data_str: str = None
 
 	# Uncompressed json
 	if json_exists:
 		with open(json_path, "r") as f:
 			data_str = f.read()
-			return json.loads(data_str)
-
 	# Compressed ddb
-	if ddb_exists:
+	elif ddb_exists:
 		with open(ddb_path, "rb") as f:
 			data_bytes = f.read()
 			data_str = zlib.decompress(data_bytes).decode()
-			return json.loads(data_str)
+
+	# Use custom decoder if provided
+	if config.custom_json_decoder is not None:
+		return config.custom_json_decoder(data_str)
+	# Otherwise, use default json decoder
+	return json.loads(data_str)
 
 
 
@@ -70,9 +76,15 @@ def unprotected_write_dict_as_json(db_name: str, db: dict):
 	json_path, json_exists, ddb_path, ddb_exists = db_paths(db_name)
 
 	# Dump db dict as string
-	db_dump = None
-	if config.pretty_json_files and not config.use_compression:
+	db_dump: str | bytes = None
+
+	# Use custom encoder if provided
+	if config.custom_json_encoder is not None and not config.use_compression:
+		db_dump = config.custom_json_encoder(db)
+	# Only generate pretty json if compression is disabled
+	elif config.pretty_json_files and not config.use_compression:
 		db_dump = json.dumps(db, indent="\t", sort_keys=True)
+	# Generate compact json
 	else:
 		db_dump = json.dumps(db)
 
@@ -80,16 +92,27 @@ def unprotected_write_dict_as_json(db_name: str, db: dict):
 	if config.use_compression:
 		if json_exists:
 			os.remove(json_path)
-		db_dump = zlib.compress(db_dump.encode(), 1)
+		db_dump = zlib.compress(db_dump if isinstance(db_dump, bytes) else db_dump.encode(), 1)
 		with open(ddb_path, "wb") as f:
 			f.write(db_dump)
+		return
 
 	# No compression is used
-	else:
-		if ddb_exists:
-			os.remove(ddb_path)
-		with open(json_path, "w") as f:
+
+	# Remove old ddb file if it exists
+	# This is necessary when switching from compression to no compression
+	if ddb_exists:
+		os.remove(ddb_path)
+
+	# If the custom encoder returns bytes, write them directly to the file
+	if isinstance(db_dump, bytes):
+		with open(json_path, "wb") as f:
 			f.write(db_dump)
+		return
+
+	# Otherwise, write the string to the file
+	with open(json_path, "w") as f:
+		f.write(db_dump)
 
 
 
